@@ -1,5 +1,5 @@
 =begin
-Copyright 2009, 2010 Andrew Horton
+Copyright 2009 to 2013, Andrew Horton
 
 This file is part of WhatWeb.
 
@@ -18,10 +18,12 @@ along with WhatWeb.  If not, see <http://www.gnu.org/licenses/>.
 =end
 
 class Output
-	def initialize(f=STDOUT)
-	# if no f, output to STDOUT, if f is a filename then open it, if f is a file use it	
+  # if no f, output to STDOUT, 
+	# if f is a filename then open it, if f is a file use it	
+	def initialize(f = STDOUT)
+	  f = STDOUT if f == "-"
 		@f = f if f.class == IO or f.class == File
-		@f=File.open(f,"a") if f.class == String
+		@f = File.open(f,"a") if f.class == String
 		@f.sync = true # we want flushed output
 	end
 
@@ -33,7 +35,7 @@ class Output
 	def suj(plugin_results)
 		suj={}
 		[:certainty, :version, :os, :string, :account, :model, :firmware, :module, :filepath].map do  |thissymbol|
-			t=plugin_results.map {|x| x[thissymbol] unless x[thissymbol].class==Regexp }.flatten.compact.sort.uniq.join(",")
+			t=plugin_results.map {|x| x[thissymbol] unless x[thissymbol].class == Regexp }.flatten.compact.sort.uniq.join(",")
 			suj[thissymbol] = t
 		end
 		suj[:certainty] = plugin_results.map {|x| x[:certainty] }.flatten.compact.sort.last.to_i # this is different, it's a number
@@ -42,11 +44,11 @@ class Output
 
 	# sort and uniq but no join. just for one plugin result
 	def sortuniq(p)
-		su={}
+		su = {}
 		[:name, :certainty, :version, :os, :string, :account, :model, :firmware, :module, :filepath].map do |thissymbol|
-			unless p[thissymbol].class==Regexp
-				t=p[thissymbol]
-				t=t.flatten.compact.sort.uniq if t.is_a?(Array)
+			unless p[thissymbol].class == Regexp
+				t = p[thissymbol]
+				t = t.flatten.compact.sort.uniq if t.is_a?(Array)
 				su[thissymbol] = t unless t.nil?
 			end					
 		end
@@ -60,8 +62,8 @@ end
 class OutputObject < Output
 	def out(target, status, results) 
 		$semaphore.synchronize do 
-			@f.puts "Identifying: "+ target.to_s
-			@f.puts "HTTP-Status: "+ status.to_s
+			@f.puts "Identifying: " + target.to_s
+			@f.puts "HTTP-Status: " + status.to_s
 			@f.puts results.pretty_inspect unless results.empty?	
 			@f.puts
 		end
@@ -69,7 +71,7 @@ class OutputObject < Output
 end
 
 class OutputVerbose < Output
-	def coloured(s,colour)
+	def coloured(s, colour)
 		use_colour = ((@f == STDOUT and $use_colour=="auto") or ($use_colour=="always"))
 
 		if use_colour
@@ -81,39 +83,37 @@ class OutputVerbose < Output
 
 	def out(target, status, results)
 		$semaphore.synchronize do
-			@f.puts "URL".ljust(7)+": #{coloured(target,'blue')}"
-			@f.puts "Status".ljust(7)+": #{status}"
-			results.sort.each do |plugin_name,plugin_results|
+			@f.puts "URL".ljust(7) + ": #{coloured(target,'blue')}"
+			@f.puts "Status".ljust(7) + ": #{status}"
+			results.sort.each do |plugin_name, plugin_results|
 				unless plugin_results.empty?
 				
-					@f.puts "   "+coloured(plugin_name,"yellow")+ " " + coloured("-"*(80-5-plugin_name.size),"dark_blue")
+					@f.puts "   " + coloured(plugin_name, "yellow") + " " + coloured("-"*(80-5-plugin_name.size), "dark_blue")
 				
-					description=[""]
+					description = [""]
 					unless Plugin.registered_plugins[plugin_name].description.nil?
-						d=Plugin.registered_plugins[plugin_name].description[0..350] 
-						d+="..." if d.size==251
-						description=word_wrap(d, 60)
+						d = Plugin.registered_plugins[plugin_name].description[0..350] 
+						d += "..." if d.size == 251
+						description = word_wrap(d, 60)
 					end
 #					@f.puts "\tCategory   : " + Plugin.registered_plugins[plugin_name].category.first unless Plugin.registered_plugins[plugin_name].category.nil?
 
 					@f.puts "\tDescription: " + description.first
-					description[1..-1].each {|line|
-						@f.puts "\t" + " "*13 + line
+					description[1..-1].each { |line|
+						@f.puts "\t" + " " * 13 + line
 					}
 
-
-					top_certainty= suj(plugin_results)[:certainty].to_i
+					top_certainty = suj(plugin_results)[:certainty].to_i
 					unless top_certainty == 100
-						@f.puts "\t"+"Certainty".ljust(11)+": " + certainty_to_words(top_certainty)
+						@f.puts "\t" + "Certainty".ljust(11) + ": " + certainty_to_words(top_certainty)
 					end
 
-					plugin_results.map {|x| sortuniq(x) }.each do |pr|
-
+					plugin_results.map { |x| sortuniq(x) }.each do |pr|
 						if pr[:name]
 							name_of_match = pr[:name]
 						else
-							name_of_match = [pr[:regexp_compiled],pr[:text],pr[:regexp].to_s,
-										pr[:ghdb],pr[:md5],pr[:tagpattern]].compact.join("|")
+							name_of_match = [pr[:regexp_compiled], pr[:text], pr[:regexp].to_s,
+										pr[:ghdb], pr[:md5], pr[:tagpattern]].compact.join("|")
 						end
 
 						pr.each do |key,value|
@@ -155,7 +155,7 @@ class OutputVerbose < Output
 							@f.puts
 						end
 
-						@f.puts "\t"+ coloured(pr.inspect.to_s,"dark_blue") if $verbose > 1
+						@f.puts "\t" + coloured(pr.inspect.to_s,"dark_blue") if $verbose > 1
 					end
 					@f.puts			
 				end
@@ -166,34 +166,40 @@ end
 
 class OutputBrief < Output
 
-def escape(s)
-	# [] and all characters up to space.
-	r=/[\[\]\001\002\003\004\005\006\a\b\t\n\v\f\r\016\017\020\021\022\023\024\025\026\027\030\031\032\e\034\035\036\037]/
-	# based on code for CGI.escape
-	s.gsub(r) do |x|
-		'%' + x.unpack('H2' * x.size).join('%').upcase
-	end
-end
+  def escape(s)
+	  # Encode all special characters # More info: http://www.asciitable.com/
+	  #r=/[^\x20-\x5A\x5E-\x7E]/
 
-# don't use colours if not to STDOUT
+	  # Encode low ascii non printable characters
+	  r=/[\x00-\x1F]/
+
+	  # based on code for CGI.escape
+	  s.gsub(r) do |x|
+		  '%' + x.unpack('H2' * x.size).join('%').upcase
+	  end
+  end
+
+  # don't use colours if not to STDOUT
 	def out(target, status, results)
 		brief_results=[]
 
-# sort results so plugins that are less important at a glance are last
-		last_plugins=%w| CSS MD5 Header-Hash Footer-Hash Tag-Hash|
-		results=results.sort_by {|x,y| last_plugins.include?(x) ? 1 : 0 }
+  # sort results so plugins that are less important at a glance are last
+  #		last_plugins=%w| CSS MD5 Header-Hash Footer-Hash Tag-Hash|
+  #		results=results.sort_by {|x,y| last_plugins.include?(x) ? 1 : 0 }
 
+		results=results.sort # sort results by plugin name alphabetically
 
 		results.each do |plugin_name,plugin_results|
 			unless plugin_results.empty?
 				suj = suj(plugin_results)
 
-				certainty, version, os, string, accounts,model,firmware,modules,filepath = suj[:certainty],suj[:version],suj[:os],suj[:string], suj[:account],suj[:model],suj[:firmware],suj[:module],suj[:filepath]
+				certainty, version, os, string, accounts,model,firmware,modules,filepath = suj[:certainty].to_i,escape(suj[:version]),escape(suj[:os]),escape(suj[:string]), escape(suj[:account]),escape(suj[:model]),escape(suj[:firmware]),escape(suj[:module]),escape(suj[:filepath])
 
+
+				# colour the output
 				# be more DRY		
 				# if plugins have categories or tags this would be better, eg. all hash plugins are grey
 				if (@f == STDOUT and $use_colour=="auto") or ($use_colour=="always")
-					 string=escape(string)
 					 coloured_string = yellow(string)
 					 coloured_string = cyan(string) if plugin_name == "HTTPServer"
  				 	 coloured_string = dark_green(string) if plugin_name == "Title"
@@ -206,24 +212,25 @@ end
  					 
 					 coloured_plugin = white(plugin_name)
 					 coloured_plugin = grey(plugin_name) if plugin_name == "MD5"
-  					 coloured_plugin = grey(plugin_name) if plugin_name == "Header-Hash"
-  					 coloured_plugin = grey(plugin_name) if plugin_name == "Footer-Hash"  					 
-  					 coloured_plugin = grey(plugin_name) if plugin_name == "CSS"
+					 coloured_plugin = grey(plugin_name) if plugin_name == "Header-Hash"
+					 coloured_plugin = grey(plugin_name) if plugin_name == "Footer-Hash"  					 
+					 coloured_plugin = grey(plugin_name) if plugin_name == "CSS"
 					 coloured_plugin = grey(plugin_name) if plugin_name == "Tag-Hash"
   					 					 
 					 p = ((certainty and certainty < 100) ? grey(certainty_to_words(certainty))+ " " : "")  +
-					   coloured_plugin + (!version.empty? ? "["+green(escape(version))+"]" : "") +
-					   (!os.empty? ? "[" + red(escape(os))+"]" : "") +	
+					   coloured_plugin + (!version.empty? ? "["+green(version)+"]" : "") +
+					   (!os.empty? ? "[" + red(os)+"]" : "") +	
 					   (!string.empty? ? "[" + coloured_string+"]" : "") +
-					   (!accounts.empty? ? "["+ cyan(escape(accounts))+"]" : "" ) +
-					   (!model.empty? ? "["+ dark_green(escape(model))+"]" : "" ) +
-					   (!firmware.empty? ? "["+ dark_green(escape(firmware))+"]" : "" ) +
-					   (!filepath.empty? ? "["+ dark_green(escape(filepath))+"]" : "" ) +
-					   (!modules.empty? ? "["+ magenta(escape(modules))+"]" : "" )
+					   (!accounts.empty? ? "["+ cyan(accounts)+"]" : "" ) +
+					   (!model.empty? ? "["+ dark_green(model)+"]" : "" ) +
+					   (!firmware.empty? ? "["+ dark_green(firmware)+"]" : "" ) +
+					   (!filepath.empty? ? "["+ dark_green(filepath)+"]" : "" ) +
+					   (!modules.empty? ? "["+ magenta(modules)+"]" : "" )
 
 					 
 					 brief_results << p
 				else
+
 					brief_results << ((certainty and certainty < 100) ? certainty_to_words(certainty)+ " " : "")  +
 					   plugin_name + (!version.empty? ? "[" + version +"]" : "") +
 					   (!os.empty? ? "[" + os+"]" : "") +
@@ -231,30 +238,46 @@ end
 					   (!accounts.empty? ? " ["+ accounts+"]" : "" ) +
 					   (!model.empty? ? "["+ model+"]" : "" ) +
 					   (!firmware.empty? ? "["+ firmware+"]" : "" ) +
-					   (!filepath.empty? ? "["+ firmware+"]" : "" ) +
+					   (!filepath.empty? ? "["+ filepath+"]" : "" ) +
 					   (!modules.empty? ? "["+ modules+"]" : "" )
 				end	
 			end
 		end
 		
-		$semaphore.synchronize do 
-			if (@f == STDOUT and $use_colour=="auto") or ($use_colour=="always")
-				@f.puts blue(target) + " [#{status}] " + brief_results.join(", ")
-			else
-				@f.puts target.to_s + " [#{status}] " + brief_results.join(", ")
-			end		
+
+		if (@f == STDOUT and $use_colour=="auto") or ($use_colour=="always")
+			brief_results_final= blue(target) + " [#{status}] " + brief_results.join(", ")
+		else
+			brief_results_final= target.to_s + " [#{status}] " + brief_results.join(", ")
+		end	
+		$semaphore.synchronize do
+			@f.puts brief_results_final
 		end
 	end
 end
 
 # XML Output #
-# Hey, do u actually use this XML output? Then I'd love to hear from you for suggestions, changes, etc.
-# Does it bother you that some types of output are joined by commas but other types aren't?
+# Does anyone use XML output?
+# We'd love to hear any suggestions you may have!
+# Does it bother you that some types of output are joined by commas
+# but other types aren't?
 class OutputXML < Output
 	def initialize(f=STDOUT)
 		super
 		@substitutions={'&'=>'&amp;', '"'=>'&quot;', '<'=>'&lt;', '>'=>'&gt;'}
-		@f.puts '<?xml version="1.0"?><?xml-stylesheet type="text/xml" href="whatweb.xsl"?>'
+
+		# only output <?xml line if it's a new file or STDOUT
+		if RUBY_VERSION =~ /^1\.8/
+			if @f == STDOUT or @f.stat.size == 0
+				@f.puts '<?xml version="1.0"?><?xml-stylesheet type="text/xml" href="whatweb.xsl"?>'
+			end
+		else
+		  # ruby 1.9
+			if @f == STDOUT or @f.size == 0
+				@f.puts '<?xml version="1.0"?><?xml-stylesheet type="text/xml" href="whatweb.xsl"?>'
+			end
+		end
+
 		@f.puts "<log>"
 	end
 
@@ -266,7 +289,17 @@ class OutputXML < Output
 	def escape(t)		
 		text=t.to_s.dup
 		# use sort_by so that & is before &quot;, etc.
-		@substitutions.sort_by {|a,b| a=="&" ? 0 : 1 }.map{|from,to| text.gsub!(from,to) }
+		@substitutions.sort_by { |a,b| a=="&" ? 0 : 1 }.map{ |from,to| text.gsub!(from,to) }
+
+		# Encode all special characters
+		# More info: http://www.asciitable.com/
+		r = /[^\x20-\x5A\x5E-\x7E]/
+
+		# based on code for CGI.escape
+		text.gsub!(r) do |x|
+			'%' + x.unpack('H2' * x.size).join('%').upcase
+		end
+
 		text
 	end
 
@@ -275,11 +308,11 @@ class OutputXML < Output
 			@f.puts "<target>"
 			@f.puts "\t<uri>#{escape(target)}</uri>"
 			@f.puts "\t<http-status>#{escape(status)}</http-status>"
-		
+
 			results.each do |plugin_name,plugin_results|		
 				@f.puts "\t<plugin>"
 				@f.puts "\t\t<name>#{escape(plugin_name)}</name>"
-			
+
 				unless plugin_results.empty?
 					# important info in brief mode is version, type and ?
 					# what's the highest probability for the match?
@@ -287,40 +320,33 @@ class OutputXML < Output
 					certainty = plugin_results.map {|x|
 		x[:certainty] unless x[:certainty].class==Regexp }.flatten.compact.sort.uniq.last
 					version = plugin_results.map {|x|
-		x[:version] unless x[:version].class==Regexp }.flatten.compact.sort.uniq.join(",")
+		x[:version] unless x[:version].class==Regexp }.flatten.compact.sort.uniq
 					os = plugin_results.map {|x|
-		x[:os] unless x[:os].class==Regexp}.flatten.compact.sort.uniq.join(",")
+		x[:os] unless x[:os].class==Regexp}.flatten.compact.sort.uniq
 					string = plugin_results.map {|x|
-		x[:string] unless x[:string].class==Regexp}.flatten.compact.sort.uniq.join(",")
+		x[:string] unless x[:string].class==Regexp}.flatten.compact.sort.uniq
 					model = plugin_results.map {|x|
-		x[:model] unless x[:model].class==Regexp}.flatten.compact.sort.uniq.join(",")
+		x[:model] unless x[:model].class==Regexp}.flatten.compact.sort.uniq
 					firmware = plugin_results.map {|x|
-		x[:firmware] unless x[:firmware].class==Regexp}.flatten.compact.sort.uniq.join(",")
+		x[:firmware] unless x[:firmware].class==Regexp}.flatten.compact.sort.uniq
 					filepath = plugin_results.map {|x|
-		x[:filepath] unless x[:filepath].class==Regexp}.flatten.compact.sort.uniq.join(",")
+		x[:filepath] unless x[:filepath].class==Regexp}.flatten.compact.sort.uniq
 
-					accounts = plugin_results.map {|x|
-		x[:account] unless x[:account].class==Regexp }.flatten.compact.sort.uniq.to_a
+					account = plugin_results.map {|x|
+		x[:account] unless x[:account].class==Regexp}.flatten.compact.sort.uniq
 					modules = plugin_results.map {|x|
-		x[:module] unless x[:module].class==Regexp}.flatten.compact.sort.uniq.to_a
-
+		x[:module] unless x[:module].class==Regexp}.flatten.compact.sort.uniq
 
+					# Output results
 					@f.puts "\t\t<certainty>#{escape(certainty)}</certainty>" if certainty and certainty < 100
 					version.map  {|x| @f.puts "\t\t<version>#{escape(x)}</version>"  }
 					os.map {|x| @f.puts "\t\t<os>#{escape(x)}</os>" }
-					string.map   {|x| @f.puts "\t\t<string>#{escape(x)}</string>" }
+					string.map {|x| @f.puts "\t\t<string>#{escape(x)}</string>" }
 					model.map {|x| @f.puts "\t\t<model>#{escape(x)}</model>" }
 					firmware.map {|x| @f.puts "\t\t<firmware>#{escape(x)}</firmware>" }
 					filepath.map {|x| @f.puts "\t\t<filepath>#{escape(x)}</filepath>" }
-
-					if accounts.size > 0
-						accounts.map {|x| @f.puts "\t\t<account>#{escape(x)}</account>" }
-						@f.puts "\t\t<accounts>\n" + accounts.map {|x| "\t\t\t<accounts>#{escape(x)}</accounts>" }.join("\n") + "\n\t\t</accounts>"
-					end
-
-					if modules.size > 0
-						@f.puts "\t\t<modules>\n" + modules.map {|x| "\t\t\t<module>#{escape(x)}</module>" }.join("\n") + "\n\t\t</modules>"
-					end
+					account.map {|x| @f.puts "\t\t<account>#{escape(x)}</account>" }
+					modules.map {|x| @f.puts "\t\t<module>#{escape(x)}</module>" }
 				end
 				@f.puts "\t</plugin>"
 			end
@@ -336,7 +362,17 @@ class OutputMagicTreeXML < Output
 	def initialize(f=STDOUT)
 		super
 		@substitutions={'&'=>'&amp;', '"'=>'&quot;', '<'=>'&lt;', '>'=>'&gt;'}
-		@f.puts '<?xml version="1.0" encoding="UTF-8"?>'
+
+		# only output <?xml line if it's a new file or STDOUT
+		if RUBY_VERSION =~ /^1\.8/
+			if @f.stat.size == 0
+				@f.puts '<?xml version="1.0" encoding="UTF-8"?>'
+			end
+		else
+			if @f.size == 0
+				@f.puts '<?xml version="1.0" encoding="UTF-8"?>'
+			end
+		end
 		@f.puts '<magictree class="MtBranchObject">'
 	end
 
@@ -346,51 +382,63 @@ class OutputMagicTreeXML < Output
 	end
 
 	def escape(t)		
-		text=t.to_s.dup
+		text = t.to_s.dup
 		# use sort_by so that & is before &quot;, etc.
-		@substitutions.sort_by {|a,b| a=="&" ? 0 : 1 }.map{|from,to| text.gsub!(from,to) }
+		@substitutions.sort_by {|a,b| a == "&" ? 0 : 1 }.map{ |from,to| text.gsub!(from,to) }
+
+		# Encode all special characters
+		# More info: http://www.asciitable.com/
+		r=/[^\x20-\x5A\x5E-\x7E]/
+
+		# based on code for CGI.escape
+		text.gsub!(r) do |x|
+			'%' + x.unpack('H2' * x.size).join('%').upcase
+		end
 		text
 	end
 
 	def out(target, status, results)
 
-		# Parse target URL and initialize host node details
-		uri = URI.parse(target.to_s)
-		@host_os=[]
-		@host_port = uri.port
-		@host_scheme = uri.scheme
+		$semaphore.synchronize do
 
-		if uri.host =~ /^[\d]{1,3}.[\d]{1,3}.[\d]{1,3}.[\d]{1,3}$/i
-			@host_ip = uri.host
-		else
-			# hostname # fqdn nodes are only used when the host IP address is not known yet
-			@host_name = uri.host
-		end
+			# Parse target URL and initialize host node details
+			uri = URI.parse(target.to_s)
+			@host_os = []
+			@host_port = uri.port
+			@host_scheme = uri.scheme
 
-		# Loop through plugin results # get host IP, country and OS
-		results.each do |plugin_name,plugin_results|
-			unless plugin_results.empty?
-				# Host IP
-				@host_ip = plugin_results.map {|x| x[:string] unless x[:string].nil?}.to_s if plugin_name =~ /^IP$/
-				# Host Country
-				@host_country = plugin_results.map {|x| x[:string] unless x[:string].nil?}.to_s if plugin_name =~ /^Country$/
-				# Host OS
-				@host_os << plugin_results.map {|x| x[:os] unless x[:os].class==Regexp}.to_s
+			# Set host node details
+			if uri.host =~ /^[\d]{1,3}\.[\d]{1,3}\.[\d]{1,3}\.[\d]{1,3}$/i
+				@host_ip = uri.host
+				@host_name = nil
+			else
+				@host_name = uri.host
+				@host_ip = nil
+			end
+
+			# Loop through plugin results # get host IP, country and OS
+			results.each do |plugin_name, plugin_results|
+				unless plugin_results.empty?
+					# Host IP
+					@host_ip = plugin_results.map {|x| x[:string] unless x[:string].nil?}.to_s if plugin_name =~ /^IP$/
+					# Host Country
+					@host_country = plugin_results.map {|x| x[:string] unless x[:string].nil?}.to_s if plugin_name =~ /^Country$/
+					# Host OS
+					@host_os << plugin_results.map {|x| x[:os] unless x[:os].class==Regexp}.to_s
+				end
 			end
-		end
 
-		$semaphore.synchronize do 
 			# testdata branch
-			@f.write '<testdata class="MtBranchObject">'
+			@f.write "<testdata class=\"MtBranchObject\"><host>#{escape(@host_ip)}"
 
-			# hostname # title attribute is not used in simple nodes  
-			@f.write "<host>#{escape(@host_ip)}<hostname>#{escape(@host_name)}</hostname></host>" unless @host_name.nil?
+			# hostname
+			@f.write "<hostname>#{escape(@host_name)}</hostname>" unless @host_name.nil?
 
 			# os
-			@host_os.compact.sort.uniq.map {|x| @f.write "<host>#{escape(@host_ip)}<os>#{escape(x.to_s)}</os></host>" unless x.empty? } unless @host_os.empty?
+			@host_os.compact.sort.uniq.map {|x| @f.write "<os>#{escape(x.to_s)}</os>" unless x.empty? } unless @host_os.empty?
 
 			# country and port nodes
-			@f.write "<host>#{escape(@host_ip)}<country>#{escape(@host_country)}</country><ipproto>tcp<port>#{escape(@host_port)}<state>open</state>"
+			@f.write "<country>#{escape(@host_country)}</country><ipproto>tcp<port>#{escape(@host_port)}<state>open</state>"
 
 			# https
 			if @host_scheme == 'https'
@@ -403,14 +451,14 @@ class OutputMagicTreeXML < Output
 			results.each do |plugin_name,plugin_results|
 
 				if !plugin_results.empty? and plugin_name !~ /^IP$/ and plugin_name !~ /^Country$/
-					certainty = plugin_results.map {|x| x[:certainty] unless x[:certainty].class==Regexp }.flatten.compact.sort.uniq.last
-					versions = plugin_results.map {|x| x[:version] unless x[:version].class==Regexp }.flatten.compact.sort.uniq.to_a
-					strings = plugin_results.map {|x| x[:string] unless x[:string].class==Regexp}.flatten.compact.sort.uniq.to_a
-					models = plugin_results.map {|x| x[:model] unless x[:model].class==Regexp}.flatten.compact.sort.uniq.to_a
-					firmwares = plugin_results.map {|x| x[:firmware] unless x[:firmware].class==Regexp}.flatten.compact.sort.uniq.to_a
-					filepaths = plugin_results.map {|x| x[:filepath] unless x[:filepath].class==Regexp}.flatten.compact.sort.uniq.to_a
-					accounts = plugin_results.map {|x| x[:account]  unless x[:account].class==Regexp }.flatten.compact.sort.uniq.to_a
-					modules = plugin_results.map {|x|  x[:module]   unless x[:module].class==Regexp}.flatten.compact.sort.uniq.to_a
+					certainty = plugin_results.map { |x| x[:certainty] unless x[:certainty].class==Regexp }.flatten.compact.sort.uniq.last
+					versions = plugin_results.map { |x| x[:version] unless x[:version].class==Regexp }.flatten.compact.sort.uniq
+					strings = plugin_results.map { |x| x[:string] unless x[:string].class==Regexp}.flatten.compact.sort.uniq
+					models = plugin_results.map { |x| x[:model] unless x[:model].class==Regexp}.flatten.compact.sort.uniq
+					firmwares = plugin_results.map { |x| x[:firmware] unless x[:firmware].class==Regexp}.flatten.compact.sort.uniq
+					filepaths = plugin_results.map { |x| x[:filepath] unless x[:filepath].class==Regexp}.flatten.compact.sort.uniq
+					accounts = plugin_results.map { |x| x[:account]  unless x[:account].class==Regexp }.flatten.compact.sort.uniq
+					modules = plugin_results.map { |x|  x[:module]   unless x[:module].class==Regexp}.flatten.compact.sort.uniq
 
 					# URL node # plugin node
 					@f.write "<url>#{escape(target)}<#{escape(plugin_name)}>"
@@ -514,7 +562,21 @@ class OutputJSON < Output
 		if obj.class == String
 #			obj=obj.upcase!
 #			obj=Iconv.iconv("UTF-8",@charset,obj).join
-			obj=obj.gsub!(/^.*$/,Iconv.iconv("UTF-8",@charset,obj).join) # this is a bad way to do this but it works			
+#pp @charset
+#pp obj.encoding
+# read this - http://blog.grayproductions.net/articles/ruby_19s_string
+			# replace invalid UTF-8 chars
+			# based on http://stackoverflow.com/a/8873922/388038
+			if String.method_defined?(:encode)
+			  obj.encode!('UTF-16', 'UTF-8', :invalid => :replace, :replace => '')
+			  obj.encode!('UTF-8', 'UTF-16')
+			end
+            obj = obj.force_encoding('UTF-8')
+
+	#	obj=obj.force_encoding("ASCII-8BIT")
+#puts obj.encoding.name
+#		obj.encode!("UTF-8",{:invalid=>:replace,:undef=>:replace})
+
 		end
 	end
 
@@ -559,13 +621,17 @@ class OutputJSON < Output
 		end
 
 		@charset=results.map {|n,r| r[0][:string] if n=="Charset" }.compact.first
+
 		unless @charset.nil? or @charset == "Failed"
 			utf8_elements!(foo) # convert foo to utf-8
 			flatten_elements!(foo)			
+		else
+			# could not find encoding force UTF-8 anyway
+			utf8_elements!(foo) 
 		end
 
 		$semaphore.synchronize do 
-			@f.puts JSON::fast_generate(foo)
+			@f.puts JSON::generate(foo)
 		end
 	end
 end
@@ -619,7 +685,7 @@ class OutputMongo < Output
 		if obj.class == String
 #			obj=obj.upcase!
 #			obj=Iconv.iconv("UTF-8",@charset,obj).join
-			obj=obj.gsub!(/^.*$/,Iconv.iconv("UTF-8",@charset,obj).join) # this is a bad way to do this but it works			
+            obj = obj.force_encoding('UTF-8')
 		end
 	end
 
@@ -637,7 +703,7 @@ class OutputMongo < Output
 
 				certainty = plugin_results.map {|x| x[:certainty] unless x[:certainty].class==Regexp }.compact.sort.uniq.last					
 				version = plugin_results.map {|x| x[:version] unless x[:version].class==Regexp }.flatten.compact.sort.uniq
-				os = plugin_results.map {|x| x[:os] unless x[:os].class=Regexp }.flatten.compact.sort.uniq
+				os = plugin_results.map {|x| x[:os] unless x[:os].class==Regexp }.flatten.compact.sort.uniq
 				string = plugin_results.map {|x| x[:string] unless x[:string].class==Regexp }.flatten.compact.sort.uniq
 				accounts = plugin_results.map {|x| x[:account] unless x[:account].class==Regexp }.flatten.compact.sort.uniq
 				model = plugin_results.map {|x| x[:model] unless x[:model].class==Regexp }.flatten.compact.sort.uniq
@@ -686,10 +752,116 @@ class OutputJSONVerbose < Output
 end
 
 class OutputErrors < Output
+	# don't need semaphore.synchronize, as it's locked by the error handling routine
 	def out(error)
-		$semaphore.synchronize do 
 			@f.puts error
+	end
+end
+
+
+class OutputSQL < Output
+	
+	def flatten_elements!(obj)
+		if obj.class == Hash
+			obj.each_value {|x| 
+				flatten_elements!(x)
+			}
 		end
-	end	
+
+		if obj.class == Array
+			obj.flatten!
+		end
+	end
+
+	def escape_for_sql(s)
+		s=s.to_s
+		if s.nil?
+			"''"
+		else
+			"'"+ s.gsub("'","\'")+"'"
+		end
+	end
+
+	def initialize(f=STDOUT)
+		super
+	end
+	
+	def create_tables
+		# feel free to modify this
+		@f.puts "CREATE TABLE plugins (plugin_id int NOT NULL AUTO_INCREMENT, name varchar(255) NOT NULL,PRIMARY KEY (plugin_id), UNIQUE (name));"
+		@f.puts "CREATE TABLE targets (target_id int NOT NULL AUTO_INCREMENT, target varchar(900) NOT NULL, status varchar(10),PRIMARY KEY (target_id), UNIQUE (target, status) );"
+		@f.puts "CREATE TABLE scans (scan_id int NOT NULL AUTO_INCREMENT, plugin_id INT NOT NULL, target_id INT NOT NULL, version varchar(255), os varchar(255), string varchar(1024), account varchar(1024), model varchar(1024), firmware varchar(1024), module varchar(1024), filepath varchar(1024), certainty varchar(10) ,PRIMARY KEY (scan_id));"
+	
+		# plugins table
+		@f.puts "INSERT INTO plugins (name) VALUES ('Custom-Plugin');"
+		@f.puts "INSERT INTO plugins (name) VALUES ('Grep');"
+
+		Plugin::registered_plugins.each do |n,p|
+			@f.puts "INSERT INTO plugins (name) VALUES (#{ escape_for_sql(n) });"
+		end
+	end
+
+	def out(target, status, results)
+
+		# nice
+		foo= {:target=>target, :http_status=>status, :plugins=>{} } 
+		
+		results.each do |plugin_name,plugin_results|		
+			thisplugin = {}
+			
+			unless plugin_results.empty?
+				# important info in brief mode is version, type and ?
+				# what's the highest probability for the match?
+
+				certainty = plugin_results.map {|x| x[:certainty] unless x[:certainty].class==Regexp }.flatten.compact.sort.uniq.last
+
+				version = plugin_results.map {|x| x[:version] unless x[:version].class==Regexp }.flatten.compact.sort.uniq
+				os = plugin_results.map {|x| x[:os] unless x[:os].class==Regexp }.flatten.compact.sort.uniq
+				string = plugin_results.map {|x| x[:string] unless x[:string].class==Regexp }.flatten.compact.sort.uniq
+				accounts = plugin_results.map {|x| x[:account] unless x[:account].class==Regexp }.flatten.compact.sort.uniq
+				model = plugin_results.map {|x| x[:model] unless x[:model].class==Regexp }.flatten.compact.sort.uniq
+				firmware = plugin_results.map {|x| x[:firmware] unless x[:firmware].class==Regexp }.flatten.compact.sort.uniq
+				modules = plugin_results.map {|x| x[:module] unless x[:module].class==Regexp }.flatten.compact.sort.uniq
+				filepath = plugin_results.map {|x| x[:filepath] unless x[:filepath].class==Regexp }.flatten.compact.sort.uniq
+
+				if !certainty.nil? and certainty != 100
+					thisplugin[:certainty] = certainty
+				end
+
+				# empty arrays
+				thisplugin[:version] = version.empty? ? [] : version
+				thisplugin[:os] = os.empty? ? [] : os
+				thisplugin[:string] = string.empty? ? [] : string
+				thisplugin[:account] = accounts.empty? ? [] : accounts
+				thisplugin[:model] = model.empty? ? [] : model
+				thisplugin[:firmware] = firmware.empty? ? [] : firmware
+				thisplugin[:module] = modules.empty? ? [] : modules
+				thisplugin[:filepath] = filepath.empty? ? [] : filepath
+
+				foo[:plugins][plugin_name.to_sym] = thisplugin
+			end
+		end
+
+		flatten_elements!(foo)			
+		
+		i_target = escape_for_sql(foo[:target])
+		insert = [escape_for_sql(foo[:http_status]), i_target].join(",")
+
+		query = "INSERT IGNORE INTO targets (status,target) VALUES (#{ insert });";
+		@f.puts query
+		foo[:plugins].each do |x| 
+
+			plugin_name = escape_for_sql(x.first.to_s)
+
+			insert = [ escape_for_sql(x[1][:version].join(",").to_s), escape_for_sql(x[1][:os].join(",").to_s),
+					escape_for_sql(x[1][:string].join(",").to_s), 	escape_for_sql(x[1][:account].join(",").to_s), escape_for_sql(x[1][:model].join(",").to_s),
+					escape_for_sql(x[1][:firmware].join(",").to_s),	escape_for_sql(x[1][:module].join(",").to_s), escape_for_sql(x[1][:filepath].join(",").to_s),
+					escape_for_sql(x[1][:certainty].to_s) ].join(",")
+
+			query = "INSERT INTO scans (target_id, plugin_id, version, os, string, account, model, firmware, module, filepath, certainty) VALUES ( (SELECT target_id from targets WHERE target = #{i_target}),(SELECT plugin_id from plugins WHERE name = #{plugin_name}), #{insert} );";
+			@f.puts query
+		end
+
+	end
 end
 
