@@ -9,13 +9,13 @@
 Plugin.define "DublinCore" do
 author "Caleb Anderson"
 version "0.1"
-description "Dublin core"
+description "Dublin Core Metadata Initiative (DCMI) supports shared innovation in metadata design and best practices across a broad range of purposes and business models. Homepage: http://dublincore.org"
+
 
-examples %w| newzealand.govt.nz www.direct.gov.uk|
 
 matches [
 {:name=>"dublin core",
-:regexp=>/<meta [^>]*name="DC\.title".*?>/i},
+:regexp=>/<meta [^>]*name="DC\.title"[^>]*>/i},
 
 ]
 
